@@ -18,8 +18,17 @@ Log::Dispatch::Jabber - Log messages via Jabber
                                                      password => "*****",
                                                      resource => "logger",
                                                     },
-	                                     to=>"someone\@a.jabber.server",
-                                             # buffer => 5,
+
+	                                     to=>["webmaster\@a.jabber.server",chief_honco\@a.jabber.server"],
+
+                                             check_presence=>1,
+
+                                             # Send a message to this address even if their
+                                             # presence indicates they are not available.
+                                             force=>"webmaster\@a.jabber.server",
+
+                                             # Buffer 5 messages before sending.
+                                             buffer => "5",
                                             );
 
  $dispatcher->add($jabber);
@@ -44,9 +53,11 @@ use strict;
 package Log::Dispatch::Jabber;
 use base qw (Log::Dispatch::Output);
 
-$Log::Dispatcher::Jabber::VERSION = '0.22';
+$Log::Dispatcher::Jabber::VERSION = '0.3';
 
-use Net::Jabber qw (Client);
+use Net::Jabber qw (Client Presence);
+
+my %presence;
 
 =head1 PACKAGE METHODS
 
@@ -148,6 +159,26 @@ Required
 
 =item *
 
+B<check_presence>
+
+Boolean.
+
+If this flag is true then a message will only be sent if a recipient's
+presence is I<normal> or I<chat>
+
+=item *
+
+B<force>
+
+A string or an array reference.
+
+A list of Jabber addresses that messages should be sent to regardless of
+their current presence status.
+
+This attribute is ignored unless the I<check_presence> attribute is true.
+
+=item *
+
 B<buffer>
 
 String. The number of messages to buffer before sending.
@@ -181,7 +212,7 @@ sub new  {
   bless $self, $class;
 
   $self->_basic_init(%args);
-  
+
   $self->{'__client'} = Net::Jabber::Client->new(
 						 debuglevel=>$args{debuglevel},
 						 debugfile=>($args{debugfile} || "stdout"),
@@ -192,10 +223,12 @@ sub new  {
     return undef;
   }
 
-  $self->{'__login'}  = $args{login};
-  $self->{'__to'}     = (ref($args{to}) eq "ARRAY") ? $args{to} : [ $args{to}];
-  $self->{'__bufto'}  = $args{buffer};
-  $self->{'__buffer'} = [];
+  $self->{'__login'}    = $args{login};
+  $self->{'__to'}       = (ref($args{to})    eq "ARRAY") ? $args{to}    : [ $args{to}];
+  $self->{'__force'}    = (ref($args{force}) eq "ARRAY") ? $args{force} : [ $args{force}];
+  $self->{'__bufto'}    = $args{buffer};
+  $self->{'__presence'} = $args{'check_presence'};
+  $self->{'__buffer'}   = [];
 
   return $self;
 }
@@ -225,42 +258,83 @@ sub log_message {
 sub _send {
   my $self = shift;
 
-  my $ok = $self->{'__client'}->Connect(
-					hostname => $self->{'__login'}->{'hostname'},
-					port     => $self->{'__login'}->{'port'},
-				       );
-
-  if (! $ok) {
-    $self->_error("Failed to connect to Jabber server:$!\n");
-    return 0;
-  }
-
-  my @auth = $self->{'__client'}->AuthSend(
-					   username => $self->{'__login'}->{'username'},
-					   password => $self->{'__login'}->{'password'},
-					   resource => $self->{'__login'}->{'resource'},
-					  );
-  
-  if ($auth[0] ne "ok") {
-    $self->_error("Failed to ident/auth with Jabber server:($auth[0]) $auth[1]. Message not sent.\n");
-    return 0;
-  }
-
   #
 
   my $im = Net::Jabber::Message->new();
   $im->SetMessage(body=>join("",@{$self->{'__buffer'}}),type=>"chat");
 
-  foreach (@{$self->{'__to'}}) {
-    $im->SetTo($_);
+  foreach my $addr (@{$self->{'__to'}}) {
+    $im->SetTo($addr);
+
+    #
+
+    my $ok = $self->{'__client'}->Connect(
+					  hostname => $self->{'__login'}->{'hostname'},
+					  port     => $self->{'__login'}->{'port'},
+					 );
+
+    if (! $ok) {
+      $self->_error("Failed to connect to Jabber server:$!\n");
+      return 0;
+    }
+
+    my @auth = $self->{'__client'}->AuthSend(
+					     username => $self->{'__login'}->{'username'},
+					     password => $self->{'__login'}->{'password'},
+					     resource => $self->{'__login'}->{'resource'},
+					    );
+
+    if ($auth[0] ne "ok") {
+      $self->_error("Failed to ident/auth with Jabber server:($auth[0]) $auth[1]. Message not sent.\n");
+      return 0;
+    }
+
+    #
+
+    if (($self->{'__presence'}) && (! grep /^($addr)$/,@{$self->{'__force'}})) {
+
+      $self->{'__client'}->SetCallBacks("presence"=>\&_presence);
+      $self->{'__client'}->PresenceSend();
+
+      unless(defined($self->{'__client'}->Process(2))) {
+	$self->_error("There was a problem with the client's connection, $!\n");
+	return 0;
+      }
+
+      unless ($presence{$addr} =~ /^(normal|chat)$/) {
+	$self->_error("Did not notify $addr : $presence{$addr}\n");
+	next;
+      }
+    }
+
+    #
+
     $self->{'__client'}->Send($im);
+    $self->{'__client'}->Disconnect();
   }
 
-  $self->{'__client'}->Disconnect();
+
   $self->{'__buffer'} = [];
   return 1;
 }
 
+# Shamelessly pilfered from the mighty mighty D.J. Adams
+# http://www.pipetree.com/jabber/extended_notify.html#Presence
+
+sub _presence {
+  my $id       = shift;
+  my $presence = shift;
+
+  if (ref($presence) ne "Net::Jabber::Presence") {
+    return undef;
+  }
+
+  # remove any resource suffix from JID
+  (my $jid = $presence->GetFrom()) =~ s!\/.*$!!;
+
+  $presence{$jid} = $presence->GetShow() || 'normal';
+}
+
 sub _error {
   my $self = shift;
 
@@ -291,11 +365,11 @@ sub DESTROY {
 
 =head1 VERSION
 
-0.22
+0.3
 
 =head1 DATE
 
-October 02, 2002
+November 25, 2002
 
 =head1 AUTHOR
 
@@ -321,10 +395,6 @@ If the package does not disconnect and does the AuthSend thing, the Jabber serve
 
 Go figure.
 
-=item *
-
-Add featuritis that will allow the package to check the presence for the list of recipients and only send messages to the first person who is online.
-
 =back
 
 =head1 BUGS
@@ -333,7 +403,80 @@ Add featuritis that will allow the package to check the presence for the list of
 
 =item *
 
-Even though you should be able to send a message to multiple recipients, and even though the debugging information seems to indicate that they are, only the first address in the I<to> lists ever gets anything.
+Sending messages to multiple recipients:
+
+I've made some progress, in a two-steps forward, one-step back kind of
+way.
+
+Specifically, I can get the package to send messages to multiple
+addresses by connecting/disconnecting for every single address sent
+instead of logging in just once for every message.
+
+Then the problem becomes that if too many notices are sent in rapid
+succession (unlikely but who I am to say) the jabberd for the sender
+will likely start to limit the connection rate and all the subsequent
+connections will fail.
+
+I've tried this with both Net::Jabber and Jabber::Connection and the
+results were the same.
+
+Ideally, I'd like to simply create one connection and send a bunch of
+messages to different addresses. I can go through the motions without
+generating any errors but the messages themselves are only ever received
+by the first address...
+
+B<As of this writing : the package may fail if you send enough messages
+in a short enough period time to freak out your jabberd.>
+
+It is recommended that you set the I<buffer> attribute in the object
+constructor.
+
+In the meantime, I'm workin' on it.
+
+ sub _send {
+    my $self = shift;
+
+    my $im = Jabber::NodeFactory->newNode("message");
+    $im->insertTag('body')->data(...);
+
+    # Where &_connect() and &_disconnect()
+    # are simply wrapper methods that DWIM 
+
+    # $self->_connect();
+    # The above works great except that only
+    # the first address in $self->{'__to'}
+    # ever receives any messages
+
+    # This would be my preferred way of doing
+    # things since there's no point in creating
+    # a gazillion connetions - unless I've spaced
+    # on some important Jabber fundamentals....
+
+    foreach my $addr (@{$self->{'__to'}}) {
+       $im->attr("to",$addr);
+
+       $self->_connect();
+       # The above works so long as not too many
+       # messages are sent in rapid succession
+
+       # Log::Dispatch::Jabber has hooks to
+       # buffer messages but if I send (4)
+       # successive notices with nothing in
+       # between, the server I'm testing against
+       # (and out-of-the-box FreeBSD port) starts
+       # to carp with 'is being connection rate limited'
+       # errors after the third notice.
+
+       # I suppose I could sleep(n) but that seems
+       # like sort of rude behaviour for a log thingy.
+
+       # Happy happy
+       $self->{'__client'}->send($im);
+       $self->_disconnect();
+    }
+
+    # $self->_disconnect()
+ }
 
 =back
 
